"use strict";

/* exported logger */

var logger = require('./logger');

var ConfigurationType = require('./ConfigurationType');

function Configuration(json) {
  var self = this;

  self._options = [];

  for (var i = 0; i < json.options.length; i++) {
    var conf = json.options[i];
    var type = conf.type;
    var value = conf.value;
    self.setOption(type, value);
  }

  var legendFiles = [];
  if (self.getOption("LENGEND_FILE_1") !== undefined) {
    legendFiles.push(self.getOption("LENGEND_FILE_1"));
  }
  if (self.getOption("LENGEND_FILE_2") !== undefined) {
    legendFiles.push(self.getOption("LENGEND_FILE_2"));
  }
  if (self.getOption("LENGEND_FILE_3") !== undefined) {
    legendFiles.push(self.getOption("LENGEND_FILE_3"));
  }
  if (self.getOption("LENGEND_FILE_4") !== undefined) {
    legendFiles.push(self.getOption("LENGEND_FILE_4"));
  }
  self.setOption(ConfigurationType.LEGEND_FILES, legendFiles);

  var overlayTypes = [];
  for (var i = 0; i < json.overlayTypes.length; i++) {
    overlayTypes.push(json.overlayTypes[i].name);
  }
  self.setOverlayTypes(overlayTypes);

  self.setImageConverters(json.imageFormats);
  self.setModelConverters(json.modelFormats);
}

Configuration.prototype.setOption = function(type, value) {
  this._options[type] = value;
};
Configuration.prototype.getOption = function(type) {
  return this._options[type];
};

Configuration.prototype.setOverlayTypes = function(overlayTypes) {
  this._overlayTypes = overlayTypes;
};
Configuration.prototype.getOverlayTypes = function() {
  return this._overlayTypes;
};

Configuration.prototype.setImageConverters = function(imageConverters) {
  this._imageConverters = imageConverters;
};
Configuration.prototype.getImageConverters = function() {
  return this._imageConverters;
};

Configuration.prototype.setModelConverters = function(modelConverters) {
  this._modelConverters = modelConverters;
};
Configuration.prototype.getModelConverters = function() {
  return this._modelConverters;
};

module.exports = Configuration;
