"use strict";

var logger = require('./logger');

/**
 * This static global object contains set of functions that returns/set data in
 * the Gui (html).
 */
var GuiConnector = {};

/**
 * X coordinate of the mouse in a browser.
 */
GuiConnector.xPos = 0;

/**
 * Y coordinate of the mouse in a browser.
 */
GuiConnector.yPos = 0;

/**
 * List of GET params passed via url.
 */

GuiConnector.init = function() {
  var self = this;

  // bootstrap tab initialization
  $("ul.nav-tabs a").click(function(e) {
    e.preventDefault();
    $(this).tab('show');
  });

  self.getParams = [];

  // find GuiConnector.getParams
  window.location.search.replace(/\??(?:([^=]+)=([^&]*)&?)/g, function() {
    function decode(s) {
      return decodeURIComponent(s.split("+").join(" "));
    }
    GuiConnector.getParams[decode(arguments[1])] = decode(arguments[2]);
  });

  // forser browser to update mouse coordinates whenever mouse move
  jQuery(document).ready(function() {
    $(document).mousemove(function(e) {
      GuiConnector.updateMouseCoordinates(e.pageX, e.pageY);
    });
  });

  if (self._windowResizeEvents === undefined) {
    self._windowResizeEvents = [];

    if (window.onresize !== null && window.onresize !== undefined) {
      self.addWindowResizeEvent(window.onresize);
    }

    window.onresize = function() {
      for (var i = 0; i < self._windowResizeEvents.length; i++) {
        self._windowResizeEvents[i]();
      }
    };
  }
};

GuiConnector.addWindowResizeEvent = function(handler) {
  this._windowResizeEvents.push(handler);
};

/**
 * Returns name of the file with LCSB logo.
 * 
 * @param bigLogo
 *          {@link Boolean} value determining if we want to have big logo or
 *          small one
 */
GuiConnector.getLcsbLogoImg = function(bigLogo) {
  if (bigLogo) {
    return 'lcsb_logo_mid.png';
  } else {
    return 'lcsb_logo.png';
  }
};

/**
 * Returns name of the file with image that should be presented when we are
 * wainting for data to be loaded.
 */
GuiConnector.getLoadingImg = function() {
  return "icons/ajax-loader.gif";
};

/**
 * Returns home directory for images in the application.
 */
GuiConnector.getImgPrefix = function() {
  return "resources/images/";
};

/**
 * Updates coordinates of the mouse in the browser.
 */
GuiConnector.updateMouseCoordinates = function(x, y) {
  this.xPos = x;
  this.yPos = y;
};

GuiConnector.alert = function(message) {
  logger.error(message);
  alert(message);
};

module.exports = GuiConnector;
