"use strict";

var Promise = require("bluebird");

/* exported logger */
var logger = require('../../logger');

var AbstractDbPanel = require('./AbstractDbPanel');
var PanelControlElementType = require('../PanelControlElementType');

function ChemicalPanel(params) {
  params.panelName = "chemical";
  AbstractDbPanel.call(this, params);

  if (params.disease===undefined) {
    this.disablePanel("DISEASE NOT DEFINED FOR PROJECT. PLEASE, DEFINE IT IN THE ADMIN SECTION.");
  }
}
ChemicalPanel.prototype = Object.create(AbstractDbPanel.prototype);
ChemicalPanel.prototype.constructor = ChemicalPanel;

ChemicalPanel.prototype.createPreamble = function(chemical) {
  var self = this;
  var result = document.createElement("div");
  if (chemical === undefined || chemical.getName() === undefined) {
    result.appendChild(self.createLabel("NOT FOUND"));
  } else {
    result.appendChild(self.createParamLine("Chemical: ", chemical.getName()));
    result.appendChild(self.createParamLine("Description: ", chemical.getDescription()));
    result.appendChild(self.createArrayParamLine("Synonyms: ", chemical.getSynonyms()));
    result.appendChild(self.createParamLine("Direct Evidence: ", chemical.getDirectEvidence()));
    result.appendChild(self.createAnnotations("Directe Evidence Publications: ", chemical.getDirectEvidenceReferences()));
    result.appendChild(self.createAnnotations("Sources: ", chemical.getReferences()));
    result.appendChild(self.createNewLine());
  }

  return result;
};

ChemicalPanel.prototype.createTableElement = function(target, icon) {
  return this.createTargetRow(target, icon);
};

ChemicalPanel.prototype.searchByQuery = function() {
  var self = this;
  var query = self.getControlElement(PanelControlElementType.SEARCH_INPUT).value;
  
  return self.getOverlayDb().searchByQuery(query);
};

ChemicalPanel.prototype.init = function() {
  var query = ServerConnector.getSessionData().getChemicalQuery();
  if (query!==undefined) {
    return this.getOverlayDb().searchByEncodedQuery(query);
  } else {
    return Promise.resolve();
  }
};

module.exports = ChemicalPanel;
