"use strict";

var Promise = require("bluebird");

/* exported logger */
var logger = require('../../logger');

var AbstractDbPanel = require('./AbstractDbPanel');
var PanelControlElementType = require('../PanelControlElementType');

function DrugPanel(params) {
  params.panelName = "drug";
  AbstractDbPanel.call(this, params);
}
DrugPanel.prototype = Object.create(AbstractDbPanel.prototype);
DrugPanel.prototype.constructor = DrugPanel;

DrugPanel.prototype.createPreamble = function(drug) {
  var self = this;
  var result = document.createElement("div");
  if (drug === undefined || drug.getName() === undefined) {
    result.appendChild(self.createLabel("NOT FOUND"));
  } else {
    result.appendChild(self.createParamLine("Drug: ", drug.getName()));
    result.appendChild(self.createParamLine("Description: ", drug.getDescription()));
    result.appendChild(self.createArrayParamLine("Synonyms: ", drug.getSynonyms()));
    result.appendChild(self.createArrayParamLine("Brand names: ", drug.getBrandNames()));
    result.appendChild(self.createParamLine("Blood brain barrier: ", drug.getBloodBrainBarrier()));
    result.appendChild(self.createAnnotations("Sources: ", drug.getReferences()));
    result.appendChild(self.createNewLine());
  }

  return result;
};

DrugPanel.prototype.createTableElement = function(target, icon) {
  return this.createTargetRow(target, icon);
};

DrugPanel.prototype.searchByQuery = function() {
  var self = this;
  var query = self.getControlElement(PanelControlElementType.SEARCH_INPUT).value;
  return self.getOverlayDb().searchByQuery(query);
};

DrugPanel.prototype.init = function() {
  var query = ServerConnector.getSessionData().getDrugQuery();
  if (query !== undefined) {
    return this.getOverlayDb().searchByEncodedQuery(query);
  } else {
    return Promise.resolve();
  }
};

module.exports = DrugPanel;
