"use strict";

/* exported logger */

var AbstractGuiElement = require('../AbstractGuiElement');
var ConfigurationType = require('../../ConfigurationType');
var Functions = require('../../Functions');
var PanelControlElementType = require('../PanelControlElementType');

var Promise = require("bluebird");

var logger = require('../../logger');

function Footer(params) {
  AbstractGuiElement.call(this, params);
  var self = this;

  self._createFooterGui();
}

Footer.prototype = Object.create(AbstractGuiElement.prototype);
Footer.prototype.constructor = Footer;

Footer.prototype._createFooterGui = function() {
  var self = this;
  var projectId = self.getMap().getProject().getProjectId();
  self.getElement().className = "footerLinks";

  var link = Functions.createElement({
    type : "a",
    name : "footerAdminLink",
    style : "padding-left:25px;",
    content : '<i class="fa fa-lock" style="font-size:17px"></i> &nbsp;ADMIN',
  });
  link.href = ServerConnector.getServerBaseUrl() + "/admin.xhtml?id=" + projectId;
  self.getElement().appendChild(link);

  link = Functions.createElement({
    type : "a",
    name : "footerManualLink",
    style : "padding-left:25px;",
    content : '<i class="fa fa-lock" style="font-size:17px"></i> &nbsp;MANUAL',
  });
  link.href = "#";
  link.onclick = function() {
    return ServerConnector.getConfigurationParam(ConfigurationType.USER_MANUAL_FILE).then(function(manualFileUrl) {
      var win = window.open(manualFileUrl, '_user_manual_');
      win.focus();
    });
  };
  self.getElement().appendChild(link);

  link = Functions.createElement({
    type : "a",
    name : "footerAdminLink",
    style : "padding-left:25px;",
    content : 'EXPORT&nbsp; <i class="fa fa-mail-forward" style="font-size:17px"></i>',
  });
  link.href = ServerConnector.getServerBaseUrl() + "/export.xhtml?id=" + projectId;
  self.getElement().appendChild(link);

  var loadingDiv = Functions.createElement({
    type : "div",
    style : "display:none; ",
  });
  loadingDiv.style.height = "100%";
  loadingDiv.style.float = "right";
  self.getElement().appendChild(loadingDiv);

  var loadingImg = Functions.createElement({
    type : "img",
    src : 'resources/images/icons/ajax-loader.gif',
  });
  loadingImg.style.height = "100%";
  loadingDiv.appendChild(loadingImg);
  this.setControlElement(PanelControlElementType.FOOTER_LOADING_DIV, loadingDiv);

};

Footer.prototype.addLoadMessage = function(message) {
  var self = this;
  self._loadMessages.push(message);
};
Footer.prototype.removeLoadMessage = function(message) {
  var self = this;
  var index = self._loadMessages.indexOf(message);
  if (index > -1) {
    self._loadMessages.splice(index, 1);
  } else {
    logger.warn("Removing message that is not there: " + message);
  }
};

Footer.prototype.init = function() {
  var self = this;
  return new Promise(function(resolve) {
    var div = self.getControlElement(PanelControlElementType.FOOTER_LOADING_DIV);

    self._loadMessages = [];
    self._onDataLoadStart = function(e) {
      self.addLoadMessage(e.arg);
      div.style.display = "block";
      div.title = e.arg;
    };

    self._onDataLoadStop = function(e) {
      self.removeLoadMessage(e.arg);
      if (self._loadMessages.length > 0) {
        div.title = self._loadMessages[0];
      } else {
        div.style.display = "none";
      }
    };
    ServerConnector.addListener("onDataLoadStart", self._onDataLoadStart);
    ServerConnector.addListener("onDataLoadStop", self._onDataLoadStop);
    resolve();
  });
};

Footer.prototype.destroy = function() {
  var self = this;
  ServerConnector.removeListener("onDataLoadStart", self._onDataLoadStart);
  ServerConnector.removeListener("onDataLoadStop", self._onDataLoadStop);
};

module.exports = Footer;
