"use strict";

/* exported logger */

var Panel = require('./Panel');
var PanelControlElementType = require('../PanelControlElementType');

var GuiConnector = require('../../GuiConnector');
var logger = require('../../logger');
var Functions = require('../../Functions');

function OverlayPanel(params) {
  params.panelName = "overlays";
  Panel.call(this, params);

  var self = this;

  $(self.getElement()).addClass("pre-scrollable");
  self._createOverlayPanelGui();

  var addButton =  this.getControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON);
 
  addButton.onclick = function(){
    self.openAddOverlayDialog();
  };

  var titleElement = this.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE);
  self.setCustomOverlaysMessage(titleElement.innerHTML);

  GuiConnector.addWindowResizeEvent(function() {
    self.onresize();
  });
  
  $("a[href='#"+self.getElement().id+"']").on('shown.bs.tab', function() {
    self.onresize();
  });
}

OverlayPanel.prototype = Object.create(Panel.prototype);
OverlayPanel.prototype.constructor = OverlayPanel;

OverlayPanel.prototype._createOverlayPanelGui = function() {
  var generalOverlaysDiv = Functions.createElement({type:"div", name:"generalOverlays", className:"searchPanel" });
  this.getElement().appendChild(generalOverlaysDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_DIV, generalOverlaysDiv);

  var generalOverlaysTitle = Functions.createElement({type:"h5", content:"GENERAL OVERLAYS:"} );
  generalOverlaysDiv.appendChild(generalOverlaysTitle);

  var generalOverlaysTableDiv = Functions.createElement({type:"table",name:"generalOverlaysTab",className:"table table-bordered", style: "width:100%"});
  generalOverlaysDiv.appendChild(generalOverlaysTableDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE, generalOverlaysTableDiv);
  
  
  var customOverlaysDiv = Functions.createElement({type:"div", name:"customOverlays", className:"searchPanel" });
  this.getElement().appendChild(customOverlaysDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_DIV, customOverlaysDiv);

  var customOverlaysTitle = Functions.createElement({type:"h5", name:"customOverlaysTitle", content:"USER-PROVIDED OVERLAYS:"} );
  customOverlaysDiv.appendChild(customOverlaysTitle);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE, customOverlaysTitle);

  var customOverlaysTableDiv = Functions.createElement({type:"table",name:"customOverlaysTab",className:"table table-bordered", style: "width:100%"});
  customOverlaysDiv.appendChild(customOverlaysTableDiv);
  this.setControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE, customOverlaysTableDiv);
  
  var centerTag = Functions.createElement({type:"center"} );
  customOverlaysDiv.appendChild(centerTag);

  var addOverlayButton = Functions.createElement({type:"button", name:"addOverlay", content:"Add overlay"} );
  centerTag.appendChild(addOverlayButton);
  this.setControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON, addOverlayButton);
};

OverlayPanel.prototype.clear = function() {
  var table = this.getControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE);
  while (table.firstChild) {
    table.removeChild(table.firstChild);
  }

  table = this.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE);
  while (table.firstChild) {
    table.removeChild(table.firstChild);
  }

};

OverlayPanel.prototype.createTableHeader = function(edit) {
  var result = document.createElement("thead");

  var row = document.createElement("tr");

  var nameTd = document.createElement("th");
  nameTd.innerHTML = "Name";
  row.appendChild(nameTd);

  var viewTd = document.createElement("th");
  viewTd.innerHTML = "View";
  row.appendChild(viewTd);

  var dataTd = document.createElement("th");
  dataTd.innerHTML = "Data";
  row.appendChild(dataTd);

  if (edit) {
    var editTd = document.createElement("th");
    editTd.innerHTML = "Edit";
    row.appendChild(editTd);
  }

  result.appendChild(row);
  return result;
};

OverlayPanel.prototype.createOverlayRow = function(overlay, checked) {
  var self = this;
  var result = document.createElement("tr");

  if (checked) {
    result.className = "active";
  }

  var nameTd = document.createElement("td");
  nameTd.innerHTML = overlay.getName();
  result.appendChild(nameTd);

  var viewTd = document.createElement("td");
  if (overlay.getInputDataAvailable()) {
    var checkbox = document.createElement("input");
    checkbox.type = "checkbox";
    checkbox.checked = checked;
    checkbox.onclick = function() {
      if (this.checked) {
        return self.getMap().openDataOverlay(overlay.getId());
      } else {
        return self.getMap().removeSelectedLayout(overlay.getId());
      }
    };
    viewTd.appendChild(checkbox);
  } else {
    var img = this.createIcon("icons/search.png");
    var link = document.createElement("a");
    link.href = "#";
    link.onclick = function() {
      self.getMap().openDataOverlay(overlay.getId());
      $(result).addClass('active').siblings().removeClass('active');
    };
    link.appendChild(img);
    viewTd.appendChild(link);
  }
  result.appendChild(viewTd);

  var dataTd = document.createElement("td");
  if (overlay.getInputDataAvailable()) {
    var button = document.createElement("button");
    button.setAttribute("name", "download-overlay-" + overlay.getId());
    button.onclick = function() {
      return ServerConnector.getOverlaySourceDownloadUrl({
        overlayId : overlay.getId()
      }).then(function(url) {
        return self.downloadFile(url);
      }).catch(GuiConnector.alert);
    };
    button.innerHTML = "<span class='ui-icon ui-icon-arrowthickstop-1-s'></span>";
    dataTd.appendChild(button);
  }

  result.appendChild(dataTd);

  if (overlay.getCreator() !== "" && overlay.getCreator() !== undefined) {
    var editTd = document.createElement("td");
    var editButton = document.createElement("button");
    editButton.setAttribute("name", "editButton");
    editButton.onclick = function() {
      self.openEditOverlayDialog(overlay);
    };
    editButton.innerHTML = "<span class='ui-icon ui-icon-document'></span>";
    editTd.appendChild(editButton);
    result.appendChild(editTd);
  }
  return result;
};

OverlayPanel.prototype.openEditOverlayDialog = function(overlay) {
  var self = this;
  var content = document.createElement("fieldset");
  content.appendChild(self.createLabel("Name"));
  var nameInput = self.createInputText(overlay.getName());
  content.appendChild(nameInput);

  content.appendChild(self.createLabel("Description"));
  var descriptionInput = self.createTextArea(overlay.getDescription());
  content.appendChild(descriptionInput);

  var buttons = [ {
    text : "SAVE",
    click : function() {
      var windowSelf = this;
      return ServerConnector.updateOverlay({
        overlayId : overlay.id,
        name : nameInput.value,
        description : descriptionInput.value
      }).then(function() {
        return self.refresh();
      }).then(function() {
        $(windowSelf).dialog("close");
      }).catch(GuiConnector.alert);
    }
  }, {
    text : "REMOVE",
    click : function() {
      self.getMap().removeSelectedLayout(overlay.getId());

      var windowSelf = this;
      return ServerConnector.removeOverlay({
        overlayId : overlay.getId()
      }).then(function() {
        return self.refresh();
      }).then(function() {
        $(windowSelf).dialog("close");
      }).catch(GuiConnector.alert);
    }
  }, {
    text : "CANCEL",
    click : function() {
      $(this).dialog("close");
    }
  } ];
  self.openDialog(content, {
    id : overlay.getId(),
    buttons : buttons,
  });
};

OverlayPanel.prototype.refresh = function() {
  var self = this;
  var user = null;

  var overlayTypes = [];
  var selectedOverlay = [];

  return ServerConnector.getOverlayTypes().then(function(types) {
    overlayTypes = types;
    return ServerConnector.getLoggedUser();
  }).then(function(loggedUser) {
    user = loggedUser;
    return self.getMap().getVisibleDataOverlays();
  }).then(function(visibleDataOverlays) {
    for (var j = 0; j < visibleDataOverlays.length; j++) {
      selectedOverlay[visibleDataOverlays[j].getId()] = true;
    }
    
    return ServerConnector.getOverlays();
  }).then(function(customOverlays) {


    var id = self.getMap().getGoogleMap().getMapTypeId().substring(2);
    selectedOverlay[id] = true;

    self.clear();

    var generalOverlays = [];
    var overlay;

    var overlays = self.getMap().getLayouts();
    for (var i = 0; i < overlays.length; i++) {
      overlay = overlays[i];
      if (overlay.getCreator() === undefined || overlay.getCreator() === "") {
        generalOverlays.push(overlay);
      }
    }

    
    var table = self.getControlElement(PanelControlElementType.OVERLAY_GENERAL_OVERLAY_TABLE);
    table.appendChild(self.createTableHeader());

    var body = document.createElement("tbody");
    table.appendChild(body);
    for (i = 0; i < generalOverlays.length; i++) {
      overlay = generalOverlays[i];
      body.appendChild(self.createOverlayRow(overlay, selectedOverlay[overlay.getId()]));
    }

    var title = self.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TITLE);
    var addButton = self.getControlElement(PanelControlElementType.OVERLAY_ADD_OVERLAY_BUTTON);
    if (user.getLogin() === "anonymous") {
      title.innerHTML = "YOU ARE NOT LOGGED IN. PLEASE, LOG IN TO UPLOAD AND VIEW CUSTOM OVERLAYS";
      addButton.style.display="none";
    } else {
      title.innerHTML = self.getCustomOverlaysMessage();
      addButton.style.display="block";

      table = self.getControlElement(PanelControlElementType.OVERLAY_CUSTOM_OVERLAY_TABLE);
      table.appendChild(self.createTableHeader(true));

      body = document.createElement("tbody");
      table.appendChild(body);
      for (i = 0; i < customOverlays.length; i++) {
        overlay = customOverlays[i];
        body.appendChild(self.createOverlayRow(overlay, selectedOverlay[overlay.getId()]));
      }
    }

    self.onresize();
    return null;
  });
};

OverlayPanel.prototype.setCustomOverlaysMessage = function(customOverlaysMessage) {
  this._customOverlaysMessage = customOverlaysMessage;
};

OverlayPanel.prototype.getCustomOverlaysMessage = function() {
  return this._customOverlaysMessage;
};

OverlayPanel.prototype.parseFile = function(fileContent) {
  var result = {};
  var lines = fileContent.split("\n");
  for (var i=0;i<lines.length;i++) {
    var line = lines[i];
    if (line.startsWith("#")) {
      if (line.indexOf("=")>0) {
        var name = line.substring(1, line.indexOf("=")).trim();
        var value = line.substring(line.indexOf("=")+1).trim();
        if (name==="NAME") {
          result.name = value;
        } else if (name==="DESCRIPTION") {
          result.description= value;
        } else if (name==="TYPE") {
          result.type= value;
        }
      } else {
        logger.warn("Invalid overlay header line: "+line);
      }
    } else {
      break;
    }
  }
  return result;
};

OverlayPanel.prototype.openAddOverlayDialog = function() {
  var self = this;
  var fileContent = null;
  var content = document.createElement("div");
  content.style.width="100%";
  content.style.height="100%";
  content.appendChild(self.createLabel("Name"));
  var nameInput = self.createInputText();
  content.appendChild(nameInput);
  content.appendChild(self.createNewLine());

  content.appendChild(self.createLabel("Description"));
  content.appendChild(self.createNewLine());
  var descriptionInput = self.createTextArea();
  content.appendChild(descriptionInput);
  content.appendChild(self.createNewLine());

  content.appendChild(self.createLabel("File"));
  var fileInput = self.createFileButton();
  fileInput.addEventListener("change", function(){
    fileContent = null;
    var file = fileInput.files[0];
    if (file) {
      var reader = new FileReader();
      reader.readAsText(file, "UTF-8");
      reader.onload = function (evt) {
        fileContent = evt.target.result;
        var data = self.parseFile(fileContent);
        if (data.name!==undefined) {
          nameInput.value =data.name; 
        } else {
          var filename = fileInput.value;
          if (filename.indexOf(".")>0) {
            filename.substr(0,filename.indexOf(".")-1);
          }
          nameInput.value =filename; 
        }
        if (data.description!==undefined) {
          descriptionInput.value =data.description; 
        }
      };
      reader.onerror = function () {
        GuiConnector.alert("Problem reading file");
      };
    }
  }, false);
  content.appendChild(fileInput);
  content.appendChild(self.createNewLine());

  var buttons = [ {
    text : "UPLOAD",
    click : function() {
      var dialog = this;
      if (fileContent===null) {
        GuiConnector.alert("No file was selected");
      } else {
        var data = {
            name : nameInput.value,
            description : descriptionInput.value,
            content : fileContent,
            filename : fileInput.value
        };          
        return ServerConnector.addOverlay(data).then(function(){
          return self.refresh();
        }).then(function(){
          $(dialog).dialog("close");
        });
      }
    }
  }, {
    text : "CANCEL",
    click : function() {
      $(this).dialog("close");
    }
  } ];
  self.openDialog(content, {
    id : "addOverlay",
    modal: true,
    buttons : buttons,
  });
};

OverlayPanel.prototype.onresize = function() {
  var self = this;
  var footerPosition = window.innerHeight;
  if (self.getParent() !== undefined) {
    footerPosition = $(self.getParent().getFooter().getElement()).offset().top;
  }

  var element = self.getElement();


  // compute the width (we can only compute it for visible elements)
  if ($(element).is(":visible")) {
    $(element).css('max-height', 10);
    
    var size = footerPosition - $(element).offset().top;
    $(element).css('max-height', size);
  } 
};


OverlayPanel.prototype.init = function() {
  return this.refresh();
};
module.exports = OverlayPanel;
