"use strict";

var Promise = require("bluebird");

/* exported logger */

var AbstractGuiElement = require('../AbstractGuiElement');
var Alias = require('../../map/data/Alias');
var IdentifiedElement = require('../../map/data/IdentifiedElement');
var Reaction = require('../../map/data/Reaction');

var Functions = require('../../functions');
var logger = require('../../logger');

function PublicationListDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.createPublicationListDialogGui();
  $(self.getElement()).dialog({
    autoOpen : false,
    resizable : false,
    width : window.innerWidth / 2,
    height : window.innerHeight / 2,
  });
}

PublicationListDialog.prototype = Object.create(AbstractGuiElement.prototype);
PublicationListDialog.prototype.constructor = PublicationListDialog;

PublicationListDialog.prototype.createPublicationListDialogGui = function() {
  var self = this;
  var head = Functions.createElement({
    type : "thead",
    content : "<tr>" + "<th>Pubmed ID</th>" + //
    "<th>Title</th>" + //
    "<th>Authors</th>" + //
    "<th>Journal</th>" + //
    "<th>Year</th>" + //
    "<th>Elements on map</th>" + //
    "</tr>"//
  });
  var body = Functions.createElement({
    type : "tbody",
  });
  var tableElement = Functions.createElement({
    type : "table",
    style : "width: 100%",
  });

  tableElement.appendChild(head);
  tableElement.appendChild(body);

  self.tableElement = tableElement;
  self.getElement().appendChild(tableElement);
};

PublicationListDialog.prototype._dataTableAjaxCall = function(data, callback) {
  var self = this;
  return ServerConnector.getPublications({
    start : data.start,
    length : data.length,
  })
      .then(
          function(publicationList) {
            var out = [];
            var allElements = [];
            for (var i = 0; i < publicationList.data.length; i++) {
              var publication = publicationList.data[i].publication;
              var elements = publicationList.data[i].elements;

              var row = [];
              row[0] = "<a href='" + publication.link + "'>" + publication.id + "</a>";
              row[1] = publication.title;
              row[2] = publication.authors.join();
              row[3] = publication.journal;
              row[4] = publication.year;
              row[5] = "<div>";
              for (var j = 0; j < elements.length; j++) {
                row[5] += "<a name='" + elements[j].id + "' href='#'>" + elements[j].type + ":" + elements[j].id
                    + "</a>, ";
                allElements.push(new IdentifiedElement(elements[j]));
              }
              row[5] += "</div>";
              out.push(row);
            }
            callback({
              draw : data.draw,
              recordsTotal : publicationList.totalSize,
              recordsFiltered : publicationList.totalSize,
              data : out,
            });
            var promises = [];
            allElements.forEach(function(element) {
              promises.push(self.getMap().getSubmapById(element.getModelId()).getModel().getByIdentifiedElement(
                  element, true).then(function(elementData) {
                var name = null;
                if (elementData instanceof Alias) {
                  name = elementData.getName();
                } else if (elementData instanceof Reaction) {
                  name = elementData.getReactionId();
                }
                $("a[name=" + elementData.getId() + "]", $(self.getElement())).html(name);
              }));
            });
            return Promise.all(promises);
          });
};

PublicationListDialog.prototype.show = function() {
  var self = this;
  $(self.getElement()).dialog("open");

  if (!$.fn.DataTable.isDataTable(self.tableElement)) {
    return new Promise(function(resolve) {
      $(self.tableElement).dataTable({
        serverSide : true,
        ordering : false,
        searching : false,
        ajax : function(data, callback, settings) {
          resolve(self._dataTableAjaxCall(data, callback, settings));
        },
      });
    });
  } else {
    return Promise.resolve();
  }

};

PublicationListDialog.prototype.destroy = function() {
  $(this.getElement()).dialog("destroy");
};

module.exports = PublicationListDialog;
