"use strict";

var Promise = require("bluebird");

/* exported logger */

var Panel = require('./Panel');
var PanelControlElementType = require('../PanelControlElementType');

var logger = require('../../logger');
var Functions = require('../../Functions');

function SubmapPanel(params) {
  params.panelName = "submap";
  Panel.call(this, params);

  var self = this;
  self._createSubmapGui();
}

SubmapPanel.prototype = Object.create(Panel.prototype);
SubmapPanel.prototype.constructor = SubmapPanel;

SubmapPanel.prototype._createSubmapGui = function() {
  var submapDiv = Functions.createElement({
    type : "div",
    name : "submapDiv",
    className : "searchPanel"
  });
  this.getElement().appendChild(submapDiv);
  this.setControlElement(PanelControlElementType.SUBMAP_DIV, submapDiv);

  var generalOverlaysTitle = Functions.createElement({
    type : "h5",
    content : "SUBMAPS:"
  });
  submapDiv.appendChild(generalOverlaysTitle);

  var submapTableDiv = Functions.createElement({
    type : "table",
    name : "submapTable",
    className : "table table-bordered",
    style : "width:100%"
  });
  submapDiv.appendChild(submapTableDiv);
  this.setControlElement(PanelControlElementType.SUBMAP_TABLE, submapTableDiv);

};

SubmapPanel.prototype.createRow = function(model) {
  var self = this;
  var result = document.createElement("tr");

  var nameTd = document.createElement("td");
  nameTd.innerHTML = model.getName();
  result.appendChild(nameTd);

  var openTd = document.createElement("td");

  if (model.getId() !== self.getMap().getId()) {
    var button = document.createElement("button");
    var img = self.createIcon("icons/search.png");
    button.appendChild(img);
    button.onclick = function() {
      self.getMap().openSubmap(model.getId());
    };
    openTd.appendChild(button);
  }

  result.appendChild(openTd);

  return result;
};

SubmapPanel.prototype.createTableHeader = function() {
  var result = document.createElement("thead");

  var row = document.createElement("tr");

  var nameTd = document.createElement("th");
  nameTd.innerHTML = "Name";
  row.appendChild(nameTd);

  var viewTd = document.createElement("th");
  viewTd.innerHTML = "View";
  row.appendChild(viewTd);

  result.appendChild(row);
  return result;
};

SubmapPanel.prototype.init = function() {
  var self = this;
  var table = self.getControlElement(PanelControlElementType.SUBMAP_TABLE);
  while (table.lastChild) {
    table.removeChild(table.lastChild);
  }
  table.appendChild(self.createTableHeader());

  table.appendChild(self.createRow(self.getMap().getModel()));

  var submodels = self.getMap().getModel().getSubmodels();
  for (var i = 0; i < submodels.length; i++) {
    table.appendChild(self.createRow(submodels[i]));
  }
  return Promise.resolve();
};

module.exports = SubmapPanel;
