"use strict";

/**
 * Class representing alias data.
 * 
 * @param javaObject
 *          object deserialized ajax query to the server side
 */
function Alias(javaObject) {
  if (javaObject.idObject !== undefined) {
    this.setId(javaObject.idObject);
  } else {
    this.setId(javaObject.id);
  }
  this.setModelId(javaObject.modelId);
  if (javaObject.bounds !== undefined) {
    this.setX(javaObject.bounds.x);
    this.setY(javaObject.bounds.y);
    this.setWidth(javaObject.bounds.width);
    this.setHeight(javaObject.bounds.height);
  }

  if (this._modelId === undefined) {
    throw new Error("ModelId is not defined for alias", javaObject);
  }

  if (javaObject.name === undefined) {
    this.setIsComplete(false);
  } else {
    this.update(javaObject);
  }
}

/**
 * Updates alias with full data information. This function should be called when
 * full informatin about alias is retrieved from server.
 * 
 * @param javaObject
 *          object representing data from server side
 */
Alias.prototype.update = function(javaObject) {
  if (javaObject.name === undefined) {
    return;
  }
  this.setDescription(javaObject.notes);
  this.setType(javaObject.type);
  this.setCharge(javaObject.charge);
  this.setSymbol(javaObject.symbol);
  this.setFullName(javaObject.fullName);
  this.setAbbreviation(javaObject.abbreviation);
  this.setFormula(javaObject.formula);
  this.setName(javaObject.name);
  this.setSynonyms(javaObject.synonyms);
  this.setFormerSymbols(javaObject.formerSymbols);
  this.setReferences(javaObject.references);
  this.setOther(javaObject.other);
  this.setHierarchyVisibilityLevel(javaObject.hierarchyVisibilityLevel);
  this.setComplexId(javaObject.complexId);
  this.setCompartmentId(javaObject.compartmentId);
  this.setIsComplete(true);
};

/**
 * Returns identifier of the alias.
 * 
 * @returns identifier of the alias
 */
Alias.prototype.getId = function() {
  return this.id;
};

Alias.prototype.setId = function(id) {
  this.id = id;
};

Alias.prototype.getFormula = function() {
  return this.formula;
};

Alias.prototype.setFormula = function(formula) {
  this.formula = formula;
};

Alias.prototype.getDescription = function() {
  return this.description;
};

Alias.prototype.setDescription = function(description) {
  this.description = description;
};

Alias.prototype.getCharge = function() {
  return this.charge;
};

Alias.prototype.setCharge = function(charge) {
  this.charge = charge;
};

Alias.prototype.getFormerSymbols = function() {
  return this.formerSymbols;
};

Alias.prototype.setFormerSymbols = function(formerSymbols) {
  this.formerSymbols = formerSymbols;
};

Alias.prototype.getOther = function(type) {
  if (this.other !== undefined) {
    return this.other[type];
  }
};

Alias.prototype.setOther = function(other) {
  this.other = other;
};

/**
 * Returns model identifier where {@link Alias} is located.
 * 
 * @returns model identifier where {@link Alias} is located
 */
Alias.prototype.getModelId = function() {
  return this._modelId;
};

Alias.prototype.setModelId = function(modelId) {
  this._modelId = modelId;
};

Alias.prototype.getType = function() {
  return this.type;
};

Alias.prototype.getX = function() {
  return this.x;
};

Alias.prototype.setX = function(x) {
  this.x = x;
};

Alias.prototype.setY = function(y) {
  this.y = y;
};

Alias.prototype.getY = function() {
  return this.y;
};

Alias.prototype.setWidth = function(width) {
  this.width = width;
};

Alias.prototype.setHeight = function(height) {
  this.height = height;
};
Alias.prototype.getWidth = function() {
  return this.width;
};

Alias.prototype.getHeight = function() {
  return this.height;
};

Alias.prototype.getName = function() {
  return this.name;
};

Alias.prototype.setName = function(name) {
  this.name = name;
};

Alias.prototype.getSynonyms = function() {
  return this.synonyms;
};

Alias.prototype.setSynonyms = function(synonyms) {
  this.synonyms = synonyms;
};

Alias.prototype.getReferences = function() {
  return this.references;
};

Alias.prototype.setReferences = function(references) {
  this.references = references;
};

Alias.prototype.getFullName = function() {
  return this.fullName;
};

Alias.prototype.setFullName = function(fullName) {
  this.fullName = fullName;
};

Alias.prototype.getSymbol = function() {
  return this.symbol;
};

Alias.prototype.setSymbol = function(symbol) {
  this.symbol = symbol;
};

Alias.prototype.getAbbreviation = function() {
  return this.abbreviation;
};

Alias.prototype.setAbbreviation = function(abbreviation) {
  this.abbreviation = abbreviation;
};

Alias.prototype.setType = function(type) {
  this.type = type;
};

Alias.prototype.isComplete = function() {
  return this._complete;
};

Alias.prototype.setIsComplete = function(complete) {
  this._complete = complete;
};

Alias.prototype.getHierarchyVisibilityLevel = function() {
  return this._hierarchyVisibilityLevel;
};

Alias.prototype.setHierarchyVisibilityLevel = function(hierarchyVisibilityLevel) {
  this._hierarchyVisibilityLevel = hierarchyVisibilityLevel;
};

Alias.prototype.getCompartmentId = function() {
  return this._compartmentId;
};

Alias.prototype.setCompartmentId = function(compartmentId) {
  this._compartmentId = compartmentId;
};
Alias.prototype.getComplexId = function() {
  return this._complexId;
};

Alias.prototype.setComplexId = function(complexId) {
  this._complexId = complexId;
};

module.exports = Alias;
