"use strict";

var AbstractMarker = require('./AbstractMarker');
var IdentifiedElement = require('../data/IdentifiedElement');

/**
 * This class describes Google maps marker (connected to {@link Alias}) that is
 * visualized on the map.
 */
function AliasMarker(element, map) {
  if (!(element instanceof IdentifiedElement)) {
    throw new Error("First argument must be of IdentifiedElement class");
  }
  AbstractMarker.call(this, element.getIcon(), map);
  // id of the alias to which marker is connected
  this.setId(element.getId());
  this.setIdentifiedElement(element);
}
AliasMarker.prototype = Object.create(AbstractMarker.prototype);
AliasMarker.prototype.constructor = AliasMarker;

/**
 * Returns identifier of this object.
 * 
 * @returns identifier of this object
 */
AliasMarker.prototype.getId = function() {
  return this._id;
};
AliasMarker.prototype.setId = function(id) {
  this._id = id;
};

/**
 * Returns {@link Alias} data for this marker.
 * 
 * @returns {@link Alias} data for this marker
 */
AliasMarker.prototype.getAliasData = function() {
  return this._aliasData;
};

/**
 * Sets {@link Alias} data for this marker.
 * 
 * @param data
 *          {@link Alias} data for this marker
 */
AliasMarker.prototype.setAliasData = function(data) {
  this._aliasData = data;
};

/**
 * Returns coordinates where marker is poiting.
 * 
 * @returns {google.maps.Point} - coordinates where marker is poiting
 */
AliasMarker.prototype.getCoordinates = function() {
  return new google.maps.Point(this._aliasData.x + this._aliasData.width / 2, this._aliasData.y
      + this._aliasData.height / 2);
};

/**
 * Function returning string marker type.
 * 
 * @returns string marker type
 */
AliasMarker.prototype.getType = function() {
  return "ALIAS";
};

AliasMarker.prototype.getModelId = function() {
  return this.getAliasData().getModelId();
};

AliasMarker.prototype.init = function() {
  var self = this;
  var model = self.getCustomMap().getModel().getSubmodelById(self.getIdentifiedElement().getModelId());
  return model.getByIdentifiedElement(self.getIdentifiedElement()).then(function(aliasData) {
    self.setAliasData(aliasData);
    return self._init();
  }).then(function() {
    self.show();
  });
};

AliasMarker.prototype.getIdentifiedElement = function() {
  return this._identifiedElement;
};

AliasMarker.prototype.setIdentifiedElement = function(identifiedElement) {
  this._identifiedElement = identifiedElement;
};

module.exports = AliasMarker;
