"use strict";

var AbstractMarker = require('./AbstractMarker');

/**
 * This class describes Google maps marker (connected to some specific point,
 * but not to any element) that is visualized on the map.
 */
function PointMarker(pointData, icon, map) {
  AbstractMarker.call(this, icon, map);
  this._id = pointData.getId();
  this._pointData = pointData;
  if (icon === null || icon === undefined) {
    throw new Error("Icon must be not null");
  }
  if (typeof map === "undefined") {
    throw new Error("Map must be passed");
  }
  this._init();
  this.show();
}
PointMarker.prototype = Object.create(AbstractMarker.prototype);
PointMarker.prototype.constructor = PointMarker;

/**
 * Returns identifier of the object.
 * 
 * @returns identifier of the object
 */
PointMarker.prototype.getId = function() {
  return this._id;
};

/**
 * Returns a {@link PointData} that is connected to this marker.
 * 
 * @returns a {@link PointData} that is connected to this marker
 */
PointMarker.prototype.getPointData = function() {
  return this._pointData;
};

PointMarker.prototype.getCoordinates = function() {
  return this._pointData.getPoint();
};

/**
 * Function returning string marker type.
 * 
 * @returns string marker type
 */
PointMarker.prototype.getType = function() {
  return "POINT";
};

PointMarker.prototype.getModelId = function() {
  return this.getPointData().getModelId();
};

module.exports = PointMarker;
