"use strict";

var Promise = require("bluebird");

var AbstractDbOverlay = require('./AbstractDbOverlay');
var ServerConnector = require('../../ServerConnector');

function CommentDbOverlay(params) {
  // call super constructor
  AbstractDbOverlay.call(this, params);

  this._elements = [];
  this._detailDataByIdentifiedElement = [];
}

CommentDbOverlay.prototype = Object.create(AbstractDbOverlay.prototype);
CommentDbOverlay.prototype.constructor = CommentDbOverlay;

CommentDbOverlay.prototype.refresh = function() {
  var self = this;
  return ServerConnector.getLightComments({}).then(function(comments) {
    self._elements = comments;
    return self._elements;
  });
};

CommentDbOverlay.prototype.clear = function() {
  this._elements = [];
  this._detailDataByIdentifiedElement = [];
  return new Promise.resolve();
};

CommentDbOverlay.prototype.getDetailDataByIdentifiedElement = function(element) {
  var self = this;
  var elementKey = element.getId() + "," + element.getType() + "," + element.getModelId();
  if (this._detailDataByIdentifiedElement[elementKey] !== undefined) {
    return Promise.resolve(this._detailDataByIdentifiedElement[elementKey]);
  } else {
    return ServerConnector.getComments({
      elementId : element.getId(),
      elementType : element.getType()
    }).then(function(comments) {
      self._detailDataByIdentifiedElement[elementKey] = comments;
      return comments;
    });
  }
};

CommentDbOverlay.prototype.getIdentifiedElements = function() {
  var self = this;
  return new Promise(function(resolve) {
    var result = [];
    for (var i = 0; i < self._elements.length; i++) {
      // we return only elements that are pinned to the map and weren't removed
      if (!self._elements[i].isRemoved()) {
        result.push(self._elements[i].getIdentifiedElement());
      }
    }
    resolve(result);
  });
};

module.exports = CommentDbOverlay;
